/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:55:29 $
#ifndef CONSTRS_H
#define CONSTRS_H

#include "re.h"
#include "fa.h"
#include "rfa.h"
#include "lbfa.h"
#include "rbfa.h"
#include "sigma.h"
#include "dfa.h"
#include <assert.h>

// Thompson's construction:
//	use the functions Thompson or Thompson_sigma, or
//	auto FA Th( E );
//	auto Reg<FA> Th2( E );

// Construction 4.5
inline FA Thompson( const RE& r ) {
	return( FA( r ) );
}

// Construction 4.3
inline FA Thompson_sigma( const RE& r ) {
	return( Reg<FA>( r ) );
}

// RFA constructions:
//	use the functions, or
//	auto RFA R( E );
//	auto Reg<RFA> R2( E );
//	To DFA:
//		R.determinism();
//		R2.determinism();
//		R.determinism2();
//		R2.determinism2();

// Definition 4.30 (variation)
inline RFA rfa( const RE& r ) {
	return( RFA( r ) );
}

// Definition 4.30
inline RFA rfa_sigma( const RE& r ) {
	return( Reg<RFA>( r ) );
}

// Construction 4.39 (variation of McNaughton-Yamada-Glushkov)
inline DFA MYG_shortcut( const RE& r ) {
	return( RFA( r ).determinism2() );
}

// Construction 4.50 (variation of Aho-Sethi-Ullman)
inline DFA ASU_shortcut( const RE& r ) {
	return( RFA( r ).determinism() );
}


// LBFA constructions:
//	using RFA's.

// Construction 4.32
inline LBFA BerrySethi( const RE& r ) {
	return( RFA( r ) );
}

// Construction 4.38
LBFA BS_variation( const RE& r ); 

// DFA from LBFA construction:
// Construction 4.39
inline DFA MYG( const RE& r ) {
	return( BerrySethi( r ).determinism() );
}

// RBFA constructions:
//	using RFAs'.

// Construction 4.45
inline RBFA BerrySethi_dual( const RE& r ) {
	return( RFA( r ) );
}

// Construction 4.48
RBFA BS_variation_dual( const RE& r );

// Construction 4.50
inline DFA ASU( const RE& r ) {
	return( BS_variation_dual( r ).determinism() );
}

#endif
