/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:56:35 $
#ifndef DFASEED_H
#define DFASEED_H

#include "state.h"
#include "stateto.h"
#include "charrang.h"
#include "crset.h"
#include "dfacomp.h"
#include <assert.h>


// A template function, representing an abstract DFA, used in the construction
// of real DFA's.
// It is assumed that class T is a DS??? class (constrained genericity). See
// dsfa.h or dsdfarev.h for examples of a DS??? class.

// Takes a T (an abstract start State), and constructs a DFA.
template<class T>
DFA_components construct_components( T& abs_start ) {

        auto DFA_components ret;

 	// Map each of the State's to a unique T, and vice-versa.
	auto StateTo<T> names;

	// Allocate a name for the new start State, and insert it into the namer.
	auto State s( ret.Q.allocate() );

	names.set_domain( ret.Q.size() );
	names.map( s ) = abs_start;

	ret.T.set_domain( ret.Q.size() );
	ret.F.set_domain( ret.Q.size() );
	// As invariant:
	//	ret.T.domain() == ret.Q.size() && ret.F.domain() == ret.Q.size()
        //	and all States < current are already done.
	auto State current;
	// Now construct the transitions, and finalness of each of the States.
	//	ret.Q grows as we go.
	for( current = 0; current < ret.Q.size(); current++ ) {
		// If the abstract state associated with current says it's final, then
		// current is too.
		if( names.lookup( current ).final() ) {
			ret.F.add( current );
		}

		// Now go through all of the out-transitions of the abstract
		// state associated with current (ie. names.lookup( current )).
		auto CRSet a( names.lookup( current ).out_labels() );

		auto CharRange b;
                auto int it;
		for( it = 0; !a.iter_end( it ); it++ ) {
			b = a.iterator( it );

                       	// Do something with the destination of the transition.
			auto T dest( names.lookup( current ).out_transition( b ) );

			// See if dest already has a name (by linear search).
			auto State i;
			for( i = 0; i < ret.Q.size() && (names.lookup( i ) != dest); i++ ) {
			}

			// The abstract state may not have a name yet, so we may need
			// to allocate one.
			if( i == ret.Q.size() ) {
				// Associate i with dest.
				// And maintain the invariant.
				auto State j( ret.Q.allocate() );

                                assert( i == j );
				names.set_domain( ret.Q.size() );
				ret.T.set_domain( ret.Q.size() );
				ret.F.set_domain( ret.Q.size() );

				names.map( i ) = dest;
			}

			// Now i is the name of dest.
			assert( dest == names.lookup( i ) );
			// Create the actual transition.
			ret.T.add_transition( current, b, i );
		}
		// That's all for current.                	
	}

	// Time to resynchronize the domain() of ret.S with ret.Q.size():
	ret.S.set_domain( ret.Q.size() );
	// Add the start State now.
	ret.S.add( s );

	// Return, constructing the DFA on the way out.
	return( ret );
}


#endif
