/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:56:54 $
#ifndef DSDEREMER_H
#define DSDEREMER_H

#include "charrang.h"
#include "crset.h"
#include "re.h"
#include "is-impl.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It represents the item sets in the DeRemer construction.
// See Construction 5.75
// DSDeRemer inherits from ISImpl for implementation.

class DSDeRemer : protected ISImpl {
public:
	// Must always have an argument-less constructor.
	inline DSDeRemer();

	// A special constructor:
	inline DSDeRemer( const RE *r );

	inline const DSDeRemer& operator=( const DSDeRemer& r );

	// The required member functions:
	inline int final() const;
	inline CRSet out_labels() const;
	DSDeRemer out_transition( const CharRange a ) const;
	inline int operator==( const DSDeRemer& r ) const;
	inline int operator!=( const DSDeRemer& r ) const;


	friend ostream& operator<<( ostream& os, const DSDeRemer& r );

	inline int class_invariant() const;

private:
	// Helpers: the optimization function of the DeRemer construction.
        inline void opt();
	void traverse_opt( const RE& r, int& i );
};

inline DSDeRemer::DSDeRemer() :
		ISImpl() {
}

inline DSDeRemer::DSDeRemer( const RE *r ) :
		ISImpl( r ) {
	assert( r->class_invariant() );
        // The ISImpl constructor will have closed before/after, but not optimized yet.
        opt();
	assert( class_invariant() );
}

inline const DSDeRemer& DSDeRemer::operator=( const DSDeRemer& r ) {
	assert( r.class_invariant() );
        ISImpl::operator=( (const ISImpl&)r );
	assert( class_invariant() );
        return( *this );
}

inline int DSDeRemer::final() const {
	assert( class_invariant() );
	return( ISImpl::final() );
}

inline CRSet DSDeRemer::out_labels() const {
	assert( class_invariant() );
        return( ISImpl::out_labels() );
}

inline int DSDeRemer::operator==( const DSDeRemer& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	return( ISImpl::operator==( r ) );
}

inline int DSDeRemer::operator!=( const DSDeRemer& r ) const {
	return( !operator==( r ) );
}              

inline void DSDeRemer::opt() {
	auto int i( 0 );
	traverse_opt( *e, i );
        assert( i == (e->num_operators()) );
}

inline int DSDeRemer::class_invariant() const {
	// Should really check that before and after are fully closed,
        // and that they are optimized.
	return( ISImpl::class_invariant() );
}

#endif
