/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:08 $
#ifndef DSDFAREV_H
#define DSDFAREV_H

#include "charrang.h"
#include "crset.h"
#include "stateset.h"
#include "dtransre.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It is used in the reversal of a DFA.
// Objects of the class represent States in the subset construction of the
// reverse of a DFA.

class DSDFARev {
public:
	// Must always have an argument-less constructor.
	inline DSDFARev();

	inline DSDFARev( const DSDFARev& r );

	// A special constructor:
	inline DSDFARev( const StateSet& rq,
			const DTransRel *rT,
			const StateSet *rS );

	inline const DSDFARev& operator=( const DSDFARev& r );

	// The required member functions:
	inline int final() const;
	CRSet out_labels() const;
	DSDFARev out_transition( const CharRange a ) const;
	inline int operator==( const DSDFARev& r ) const;
	inline int operator!=( const DSDFARev& r ) const;

	friend ostream& operator<<( ostream& os, const DSDFARev& r );

	inline int class_invariant() const;

private:
	StateSet which;
	const DTransRel *T;
	const StateSet *S;
};

// Must always have an argument-less constructor.
inline DSDFARev::DSDFARev() :
		which(),
		T( 0 ),
		S( 0 ) {}

inline DSDFARev::DSDFARev( const  DSDFARev& r ) :
		which( r.which ),
		T( r.T ),
		S( r.S ) {
	assert( class_invariant() );
}

// A special constructor:
inline DSDFARev::DSDFARev( const StateSet& rq,
		const DTransRel *rT,
		const StateSet *rS ) :
			which( rq ),
			T( rT ),
			S( rS ) {
	assert( class_invariant() );
}

inline const DSDFARev& DSDFARev::operator=( const DSDFARev& r ) {
	assert( r.class_invariant() );
	which = r.which;
	T = r.T;
	S = r.S;
	assert( class_invariant() );
        return( *this );
}

inline int DSDFARev::final() const {
	assert( class_invariant() );
	return( which.not_disjoint( *S ) );
}

inline int DSDFARev::operator==( const DSDFARev& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
        assert( T == r.T && S == r.S );
	return( which == r.which );
}

inline int DSDFARev::operator!=( const DSDFARev& r ) const {
	return( !operator==( r ) );
}

inline int DSDFARev::class_invariant() const {
	return( T != 0
		&& S != 0
		&& which.domain() == T->domain()
		&& which.domain() == S->domain() );
}

#endif
