/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:23 $
#ifndef DSFA_H
#define DSFA_H

#include "charrang.h"
#include "crset.h"
#include "stateset.h"
#include "staterel.h"
#include "transrel.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It is used in the construction of a DFA from an FA. The behaviour
// of a DSFA follows the simulation behaviour of an FA as in the subset construction.

class DSFA {
public:
	// Must always have an argument-less constructor.
	inline DSFA();

	inline DSFA( const DSFA& r );

	// A special constructor:
	DSFA( const StateSet& rq,
			const TransRel *rT,
                        const StateRel *rE,
			const StateSet *rF );

	inline const DSFA& operator=( const DSFA& r );

	// The required member functions:
	inline int final() const;
	CRSet out_labels() const;
	DSFA out_transition( const CharRange a ) const;
	inline int operator==( const DSFA& r ) const;
	inline int operator!=( const DSFA& r ) const;


	friend ostream& operator<<( ostream& os, const DSFA& r );

	inline int class_invariant() const;

private:
	StateSet which;
	const TransRel *T;
        const StateRel *E;
	const StateSet *F;
};

// Must always have an argument-less constructor.
inline DSFA::DSFA() :
		which(),
		T( 0 ),
		E( 0 ),
		F( 0 ) {
	// No assertion of the class invariant, since it won't qualify.
}

inline DSFA::DSFA( const DSFA& r ) :
		which( r.which ),
		T( r.T ),
		E( r.E ),
		F( r.F ) {
	assert( class_invariant() );
}

inline const DSFA& DSFA::operator=( const DSFA& r ) {
	assert( r.class_invariant() );
	// *this may not satisfy its invariant yet.
	which = r.which;
	T = r.T;
        E = r.E;
	F = r.F;
	assert( class_invariant() );
        return( *this );
}

inline int DSFA::final() const {
	assert( class_invariant() );
	return( which.not_disjoint( *F ) );
}

inline int DSFA::operator==( const DSFA& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
        assert( T == r.T && E == r.E && F == r.F );
	return( which == r.which );
}

inline int DSFA::operator!=( const DSFA& r ) const {
	return( !operator==( r ) );
}

inline int DSFA::class_invariant() const {
	return( T != 0
		&& E != 0
                && F != 0
		&& which.domain() == T->domain()
		&& which.domain() == E->domain()
		&& which.domain() == F->domain()
		&& which == E->closure( which ) );
}

#endif
