/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:27 $
#ifndef DSIS_OPT_H
#define DSIS_OPT_H

#include "charrang.h"
#include "crset.h"
#include "re.h"
#include "is-impl.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It represents the item sets in the DeRemer construction.
// See Construction 5.75
// DSIS_opt inherits from ISImpl for implementation.

class DSIS_opt : protected ISImpl {
public:
	// Must always have an argument-less constructor.
	inline DSIS_opt();

	// A special constructor:
	inline DSIS_opt( const RE *r );

	inline const DSIS_opt& operator=( const DSIS_opt& r );

	// The required member functions:
	inline int final() const;
	inline CRSet out_labels() const;
	DSIS_opt out_transition( const CharRange a ) const;
	inline int operator==( const DSIS_opt& r ) const;
	inline int operator!=( const DSIS_opt& r ) const;


	friend ostream& operator<<( ostream& os, const DSIS_opt& r );

	inline int class_invariant() const;

private:
	// Helpers: the optimization function of the DeRemer construction.
        inline void opt();
	void traverse_opt( const RE& r, int& i );
};

inline DSIS_opt::DSIS_opt() :
		ISImpl() {
}

inline DSIS_opt::DSIS_opt( const RE *r ) :
		ISImpl( r ) {
	assert( r->class_invariant() );
        // The ISImpl constructor will close before/after, but not optimize.
        opt();
	assert( class_invariant() );
}

inline const DSIS_opt& DSIS_opt::operator=( const DSIS_opt& r ) {
	assert( r.class_invariant() );
        ISImpl::operator=( (const ISImpl&)r );
	assert( class_invariant() );
        return( *this );
}

inline int DSIS_opt::final() const {
	assert( class_invariant() );
	return( ISImpl::final() );
}

inline CRSet DSIS_opt::out_labels() const {
	assert( class_invariant() );
        return( ISImpl::out_labels() );
}

inline int DSIS_opt::operator==( const DSIS_opt& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	return( ISImpl::operator==( r ) );
}

inline int DSIS_opt::operator!=( const DSIS_opt& r ) const {
	return( !operator==( r ) );
}              

inline void DSIS_opt::opt() {
	// Remember if there's a dot after the root.
	auto int par( after.contains( 0 ) );
        after.clear();
	auto int i( 0 );
	traverse_opt( *e, i );
	assert( i == (e->num_operators()) );
	// The dot after the root may have been removed, so it can be put back.
	if( par ) {
        	after.set_bit( 0 );
        }
}

inline int DSIS_opt::class_invariant() const {
	// Should really check that before and after are fully closed,
        // and that they are optimized.
	return( ISImpl::class_invariant() );
}

#endif
