/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:31 $
#ifndef DSIS_H
#define DSIS_H

#include "charrang.h"
#include "crset.h"
#include "re.h"
#include "is-impl.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It represents the item sets in the Iconstr construction.
// See Construction 5.69
// DSIS inherits from ISImpl for implementation.

class DSIS : protected ISImpl {
public:
	// Must always have an argument-less constructor.
	inline DSIS();

	// A special constructor:
	inline DSIS( const RE *r );

	inline const DSIS& operator=( const DSIS& r );

	// The required member functions:
	inline int final() const;
	inline CRSet out_labels() const;
	DSIS out_transition( const CharRange a ) const;
	inline int operator==( const DSIS& r ) const;
	inline int operator!=( const DSIS& r ) const;


	friend ostream& operator<<( ostream& os, const DSIS& r );

	inline int class_invariant() const;
};

// The default constructor won't leave *this in a condition satisfying the
// class invariant.
inline DSIS::DSIS() :
		ISImpl() {
}

inline DSIS::DSIS( const RE *r ) :
		ISImpl( r ) {
	assert( r->class_invariant() );
	assert( class_invariant() );
}

inline const DSIS& DSIS::operator=( const DSIS& r ) {
	assert( r.class_invariant() );
        ISImpl::operator=( (const ISImpl&)r );
	// *this may not satisfy the invariant until after the assignment.
	assert( class_invariant() );
        return( *this );
}

inline int DSIS::final() const {
	assert( class_invariant() );
	return( ISImpl::final() );
}

inline CRSet DSIS::out_labels() const {
	assert( class_invariant() );
        return( ISImpl::out_labels() );
}

inline int DSIS::operator==( const DSIS& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	return( ISImpl::operator==( r ) );
}

inline int DSIS::operator!=( const DSIS& r ) const {
	return( !operator==( r ) );
}              

inline int DSIS::class_invariant() const {
	// Should really check that before and after are fully closed.
	return( ISImpl::class_invariant() );
}

#endif
