/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:38 $
#ifndef DSRBFA_H
#define DSRBFA_H

#include "charrang.h"
#include "crset.h"
#include "state.h"
#include "stateset.h"
#include "staterel.h"
#include "transrel.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It is used in the construction of a DFA from an RBFA. The behaviour
// of a DSRBFA follows the simulation behaviour of an RBFA.

class DSRBFA {
public:
	// Must always have an argument-less constructor.
	inline DSRBFA();

	inline DSRBFA( const DSRBFA& r );

	// A special constructor:
	DSRBFA( const StateSet& rq,
			const Trans *rQmap_inverse,
                        const StateRel *rfollow,
			const State rf );

	inline const DSRBFA& operator=( const DSRBFA& r );

	// The required member functions:
	inline int final() const;
	CRSet out_labels() const;
	DSRBFA out_transition( const CharRange a ) const;
	inline int operator==( const DSRBFA& r ) const;
	inline int operator!=( const DSRBFA& r ) const;


	friend ostream& operator<<( ostream& os, const DSRBFA& r );

	inline int class_invariant() const;

private:
	StateSet which;
	const Trans *Qmap_inverse;
	const StateRel *follow;
	State f;
};

// Must always have an argument-less constructor.
inline DSRBFA::DSRBFA() :
		which(),
		Qmap_inverse( 0 ),
		follow( 0 ),
		f( Invalid ) {
	// *this will not yet satisfy the class invariant.
}

inline DSRBFA::DSRBFA( const DSRBFA& r ) :
		which( r.which ),
		Qmap_inverse( r.Qmap_inverse ),
		follow( r.follow ),
		f( r.f ) {
	assert( class_invariant() );
}

inline const DSRBFA& DSRBFA::operator=( const DSRBFA& r ) {
	assert( r.class_invariant() );
	// *this may not satisfy the invariant yet.
	which = r.which;
	Qmap_inverse = r.Qmap_inverse;
        follow = r.follow;
	f = r.f;
	assert( class_invariant() );
        return( *this );
}

inline int DSRBFA::final() const {
	assert( class_invariant() );
	return( which.contains( f ) );
}

inline int DSRBFA::operator==( const DSRBFA& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
        assert( Qmap_inverse == r.Qmap_inverse && follow == r.follow && f == r.f );
	return( which == r.which );
}

inline int DSRBFA::operator!=( const DSRBFA& r ) const {
	return( !operator==( r ) );
}              

inline int DSRBFA::class_invariant() const {
	return( Qmap_inverse != 0
		&& follow != 0
		&& 0 <= f
		&& f < follow->domain()
		&& which.domain() == Qmap_inverse->range()
		&& which.domain() == follow->domain() );
}

#endif
