/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:41 $
#ifndef DSREOPT_H
#define DSREOPT_H

#include "charrang.h"
#include "crset.h"
#include "re.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It is used in the construction of a DFA from an RE (using
// Brzozowski's optimized construction). It differs from DSRE in that the derivatives
// are optimized (redundant information is removed -- see deriv.cpp)
// Most member functions are calls through to the corresponding RE members.
// DSREopt inherits from RE for implementation.

class DSREopt : protected RE {
public:
	// Must always have an argument-less constructor.
	inline DSREopt();

	// A special constructor:
	inline DSREopt( const RE& r );

	inline const DSREopt& operator=( const DSREopt& r );

	// The required member functions:
	inline int final() const;
	inline CRSet out_labels() const;
	inline DSREopt out_transition( const CharRange a ) const;
	inline int operator==( const DSREopt& r ) const;
	inline int operator!=( const DSREopt& r ) const;


	friend ostream& operator<<( ostream& os, const DSREopt& r );

	inline int class_invariant() const;
};

// Must always have an argument-less constructor.
inline DSREopt::DSREopt() :
		RE() {
	assert( class_invariant() );
}

inline DSREopt::DSREopt( const RE& r ) :
		RE( r ) {
	assert( r.class_invariant() );
	// They must always be in similarity normal form SNF (see deriv.cpp)
	RE::snf();
        // They must also be reduced.
        RE::reduce();
        assert( class_invariant() );
}

inline const DSREopt& DSREopt::operator=( const DSREopt& r ) {
	assert( r.class_invariant() );
        RE::operator=( (const RE&)r );
	assert( class_invariant() );
        return( *this );
}

inline int DSREopt::final() const {
	assert( class_invariant() );
	return( RE::Null() );
}

inline CRSet DSREopt::out_labels() const {
	assert( class_invariant() );
        return( RE::First() );
}

inline DSREopt DSREopt::out_transition( const CharRange a ) const {
	assert( class_invariant() );
        return( RE::derivative( a ) );
}

inline int DSREopt::operator==( const DSREopt& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	return( RE::operator==( r ) );
}

inline int DSREopt::operator!=( const DSREopt& r ) const {
	return( !operator==( r ) );
}              

inline int DSREopt::class_invariant() const {
	return( RE::class_invariant() && RE::in_snf() );
}

#endif
