/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:44 $
#ifndef DSRE_H
#define DSRE_H

#include "charrang.h"
#include "crset.h"
#include "re.h"
#include <assert.h>
#include <iostream.h>


// This class is used to represent abstract States in a DFA that is still under
// construction. It is used in the construction of a DFA from an RE (using
// Brzozowski's construction). Most member functions are calls through to the
// corresponding RE members.
// DSRE inherits from RE for implementation.

class DSRE : protected RE {
public:
	// Must always have an argument-less constructor.
	inline DSRE();

	// A special constructor:
	inline DSRE( const RE& r );

	inline const DSRE& operator=( const DSRE& r );

	// The required member functions:
	inline int final() const;
	inline CRSet out_labels() const;
	inline DSRE out_transition( const CharRange a ) const;
	inline int operator==( const DSRE& r ) const;
	inline int operator!=( const DSRE& r ) const;


	friend ostream& operator<<( ostream& os, const DSRE& r );

	inline int class_invariant() const;
};

// Must always have an argument-less constructor.
inline DSRE::DSRE() :
		RE() {
	assert( class_invariant() );
}

inline DSRE::DSRE( const RE& r ) :
		RE( r ) {
	assert( r.class_invariant() );
	// They must always be in similarity normal form SNF (see deriv.cpp)
	RE::snf();
        assert( class_invariant() );
}

inline const DSRE& DSRE::operator=( const DSRE& r ) {
	assert( r.class_invariant() );
        RE::operator=( (const RE&)r );
	assert( class_invariant() );
        return( *this );
}

inline int DSRE::final() const {
	assert( class_invariant() );
	return( RE::Null() );
}

inline CRSet DSRE::out_labels() const {
	assert( class_invariant() );
        return( RE::First() );
}

inline DSRE DSRE::out_transition( const CharRange a ) const {
	assert( class_invariant() );
        return( RE::derivative( a ) );
}

inline int DSRE::operator==( const DSRE& r ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	return( RE::operator==( r ) );
}

inline int DSRE::operator!=( const DSRE& r ) const {
	return( !operator==( r ) );
}              

inline int DSRE::class_invariant() const {
	return( RE::class_invariant() && RE::in_snf() );
}

#endif
