/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:57:55 $
#ifndef DTRANS_H
#define DTRANS_H

#include "charrang.h"
#include "crset.h"
#include "state.h"
#include "stateset.h"
#include "tr-pair.h"
#include "tr-impl.h"
#include <assert.h>
#include <iostreams.h>

// Implement a DFA's transition function for one State.
// Map a char to a unique next State, using TransImpl

class DTrans : protected TransImpl {
public:
	// Constructors, destructors, operator=:

	// By default, don't introduce any transitions.
	inline DTrans();

        // Copy constructor does a dynamic memory copy.
	inline DTrans( const DTrans& r );

	inline const DTrans& operator=( const DTrans& r );


	// Normal member functions:

	// Map a char to the unique next state.
	State operator[]( const char a ) const;

	// Map a CharRange to the corresponding State
	//	assuming that the CharRange is entirely contained in the label
	//	of a transition.
	State range_transition( const CharRange a ) const;

	// What are the labels of transitions out of *this.
	inline CRSet out_labels() const;

	// What are all transition labels into StateSet r?
	inline CRSet labels_into( const StateSet& r ) const;

	// Is there a valid out-transition on a?
	int valid_out_transition( const CharRange a ) const;

	// What is the range (States) of this map?
	//	return a StateSet with domain dom.
	StateSet range( int dom ) const;


	// Special member functions:

	// Recycle this entire structure.
	inline void reincarnate();

	// Create a new out-transition.
	inline DTrans& add_transition( const CharRange a, const State q );

	friend ostream& operator<<( ostream& os, const DTrans& r );

	inline int class_invariant() const;
};


// Inlines (mostly calling the base class):
inline DTrans::DTrans() :
		TransImpl() {
	assert( class_invariant() );
}

inline DTrans::DTrans( const DTrans& r ) :
		TransImpl( (TransImpl&)r ) {
	assert( class_invariant() );
}

inline const DTrans& DTrans::operator=( const DTrans& r ) {
	assert( class_invariant() );
	assert( r.class_invariant() );
	TransImpl::operator=( (TransImpl&)r );
        assert( class_invariant() );
        return( *this );
}

inline CRSet DTrans::out_labels() const {
	assert( class_invariant() );
	return( TransImpl::out_labels() );
}

inline CRSet DTrans::labels_into( const StateSet& r ) const {
	assert( class_invariant() );
        return( TransImpl::labels_into( r ) );
}

inline void DTrans::reincarnate() {
	assert( class_invariant() );
	TransImpl::reincarnate();
}

inline DTrans& DTrans::add_transition( const CharRange a, const State q ) {
	assert( class_invariant() );
	TransImpl::add_transition( a, q );
        return( *this );
}

inline int DTrans::class_invariant() const {
	// Should also check that all transitions are on different symbols.
	return( TransImpl::class_invariant() );
}

#endif
