/* (c) Copyright 1994 by Bruce Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:58:05 $
#ifndef FA_H
#define FA_H

#include "faabs.h"
#include "st-pool.h"
#include "stateset.h"
#include "staterel.h"
#include "transrel.h"
#include "re.h"
#include <assert.h>
#include <iostream.h>

// Implement general finite automata (Definition 2.1 of the Taxonomy).
// These can be constructed by the Sigma-algebra (see sig-fa.cpp) or using
// the special constructor from RE's.

class FA : virtual public FAabs {
public:
	// Constructors, destructors, operator=:
	FA();

	// Provide a copy constructor:
	FA( const FA& r );

	// A special constructor, implementing the Thompson's top-down construction
	// (see Construction 4.5).
	FA( const RE& e );

	// Default operator= is okay.

	// Basic FAabs member functions (without overriding acceptable()):

        virtual int num_states() const;
	virtual void restart();
	virtual void advance( char a );
	virtual int in_final() const;
	virtual int stuck();
	virtual DFA determinism() const;


	// Special member functions:

	friend ostream& operator<<( ostream& os, const FA& r );

	inline int class_invariant() const;

protected:
	// Functions states_reqd, td (see Construction 4.5) for use in the constructor from RE.
	int states_reqd( const RE& e );
	void td( const State s, const RE& e, const State f );

	// recycle this FA:
	void reincarnate();

	// Implementation details:

	StatePool Q;
	StateSet S, F;

	// Transitions maps each State to its out-transitions.
	TransRel Transitions;
        // E is the epsilon transition relation.
	StateRel E;

	// Simulation stuff:
        StateSet current;
};

inline int FA::class_invariant() const {
	// Should also check that current is E-closed.
	return( Q.size() == S.domain()
		&& Q.size() == F.domain()
		&& Q.size() == Transitions.domain()
		&& Q.size() == E.domain()
		&& Q.size() == current.domain()
		&& S.class_invariant()
		&& F.class_invariant()
		&& Transitions.class_invariant()
		&& E.class_invariant()
		&& current.class_invariant() );
}

#endif

