/* (c) Copyright 1994 by Bruce Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:58:07 $
#ifndef FAABS_H
#define FAABS_H

// Just mention DFA here, since circularity would be a problem if we include dfa.h.
class DFA;

// Abstract finite automaton. All other automaton types are derived from this one.
// Basic FA operations are supported.

class FAabs {
public:
	// Return the number of states (or some reasonably close measure).
	virtual int num_states() const = 0;

	// Reset the current state before beginning to process a string.
        // This is not the default condition for most of the derived classes.
	virtual void restart() = 0;

        // Advance the current state by processing a character.
	virtual void advance( char a ) = 0;

        // Is the current state an accepting (final) one?
	virtual int in_final() const = 0;

        // Is the automaton stuck?
	virtual int stuck() = 0;

        // Is the string w acceptable?
	virtual int acceptable( const char *w ) {
		for( restart(); !stuck() && *w; advance( *(w++) ) ) {}
		// It's acceptable if *this is final, and the whole of w was consumed.
		return( in_final() && !*w );
	}

	// Return a DFA accepting the same language as *this.
	virtual DFA determinism() const = 0;
};

#endif

