/* (c) Copyright 1994 by Bruce Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:58:54 $
#ifndef LBFA_H
#define LBFA_H

#include "state.h"
#include "faabs.h"
#include "st-pool.h"
#include "stateset.h"
#include "staterel.h"
#include "trans.h"
#include "rfa.h"
#include <assert.h>
#include <iostream.h>

// Implement left-biased FA's (see Definition 4.20 of the Taxonomy).

class LBFA : virtual public FAabs {
public:
	// Constructors, destructors, operator=.

	// Default constructor must build the empty language acceptor.
	LBFA();

	// Default copy constructor, destr, operator= are okay.

        // Specially construct from an RFA (see Definition 4.28).
	LBFA( const RFA& r );


	// Standard FAabs operators. Don't override acceptable():

        virtual int num_states() const;
	virtual void restart();
	virtual void advance( char a );
	virtual int in_final() const;
	virtual int stuck();
	virtual DFA determinism() const;


	// Some Sigma-algebra conversions:
	//	taken from Definitions 4.28 and 4.35
	virtual LBFA& decode( const RFA& r );
	virtual LBFA& convert( const RFA& r );


	// Special member functions:

        friend ostream& operator<<( ostream& os, const LBFA& r );

	inline int class_invariant() const;

protected:
	// Implementation stuff, protected for Reg<LBFA>:

	StatePool Q;
        // Single start state.
        State s;
	StateSet F;
        // Qmap (see Definition 4.24) stored as its inverse.
	Trans Qmap_inverse;
	StateRel follow;

	// Simulation stuff:
        StateSet current;
};

inline int LBFA::class_invariant() const {
	return( Q.contains( s )
		&& Q.size() == F.domain()
                && Q.size() == Qmap_inverse.range()
		&& Q.size() == follow.domain()
		&& Q.size() == current.domain()
		&& F.class_invariant()
		&& Qmap_inverse.class_invariant()
		&& follow.class_invariant()
		&& current.class_invariant() );
}

#endif
