/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:03 $
#include "crset.h"
#include "state.h"
#include "stateset.h"
#include "st-eqrel.h"
#include "dfa.h"

// Implement Algorithm 4.8 (Hopcroft's O(n log n) algorithm).
DFA& DFA::min_Hopcroft() {
	assert( class_invariant() );

	// This algorithm requires that the DFA not have any final unreachable
	// States.
	assert( Usefulf() );

	auto State q;

	// Keep track of the combination of all of the out labels of State's.
	auto CRSet C;
	for( q = 0; q < Q.size(); q++ ) {
		C.combine( T.out_labels( q ) );
	}

	// Encode set L as a mapping from State to [0,|C|] where:
	//	if q is a representative of a class in the partition P, then
	//	L (the abstract list) contains
	//		([q],C_0), ([q],C_1), ..., ([q],C_(L(q)-1))
	//	but not
	//		([q],C_(L(q))), ..., ([q],C_(|C|-1))
	auto int *const L( new int [Q.size()] );
	for( q = 0; q < Q.size(); q++ ) {
		L[q] = 0;
	}

	// Initialize P to be the total equivalence relation.
	auto StateEqRel P( Q.size() );
	// Now set P to be E_0.
	P.split( F );

	// Now, build the set of representatives and initialize L.
	auto StateSet repr( P.representatives() );
	if( F.size() <= (Q.size() - F.size()) ) {
		repr.intersection( F );
	} else {
		repr.remove( F );
	}

	// Do the final set up of L
	for( repr.iter_start( q ); !repr.iter_end( q ); repr.iter_next( q ) ) {
		L[q] = C.size();
	}

	// Use a break to get of this loop.
	while( 1 ) {
		// Find the first pair in L that still needs processing.
		for( q = 0; q < Q.size() && !L[q]; q++ );

		// It may be that we're at the end of the processing.
		if( q == Q.size() ) {
			break;
		} else {
			// Mark this element of L as processed.
			L[q]--;

			// Iterate over all eq. classes, and try to split them.
			auto State p;
			repr = P.representatives();
			for( repr.iter_start( p ); !repr.iter_end( p );
					repr.iter_next( p ) ) {
				// Now split [p] w.r.t. (q,C_(L[q]))
				auto State r( split( p, q, C.iterator( L[q] ), P ) );
				// r is the representative of the new split of the
				// eq. class that was represented by p.

				if( r != Invalid ) {
					// p and r are the new representatives.
					// Now update L with the smallest of
					// [p] and [r].
					if( P.equiv_class( p ).size()
							<= P.equiv_class( r ).size() ) {
						L[r] = L[p];
						L[p] = C.size();
					} else {
						L[r] = C.size();
					} // if
				} // if
			} // for
		} // if
	} // while

	// L is no longer needed.
	delete L;

	// We can now use P to compress the DFA.
	compress( P );

	assert( class_invariant() );
	return( *this );
}

