/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:06 $
#include "charrang.h"
#include "state.h"
#include "stateset.h"
#include "st-eqrel.h"
#include "dfa.h"

State DFA::split( const State p, const State q, const CharRange a, StateEqRel& P ) const {
	assert( class_invariant() );
        assert( P.class_invariant() );

        // p and q must be representatives of their eq. classes.
	assert( p == P.eq_class_representative( p ) );
	assert( q == P.eq_class_representative( q ) );

	// Split [p] with respect to [q] and CharRange a.
	auto StateSet part;
	part.set_domain( Q.size() );

	// Iterate over [p], and see whether each member transitions into [q] 
	// on CharRange a.
	auto State st;
	for( P.equiv_class( p ).iter_start( st );
			!P.equiv_class( p ).iter_end( st );
			P.equiv_class( p ).iter_next( st ) ) {
		auto State dest( T.transition_on_range( st, a ) );
		// It could be that dest == Invalid.
		//	if not, check if dest is in [q].
		if( dest != Invalid && P.equivalent( dest, q ) ) {
                	part.add( st );
		} 
	}

	// The following containment must hold after the splitting.
	assert( P.equiv_class( p ).contains( part ) );

	// Return non-zero if something was split.
	if( (part != P.equiv_class( p )) && !part.empty() ) {
		// Now figure out what the other piece is.
		auto StateSet otherpiece( P.equiv_class( p ) );
		otherpiece.remove( part );
		assert( !otherpiece.empty() );

		P.split( part );
		assert( class_invariant() );

		// Now, we must return the representative of the newly created
		// equivalence class.
		auto State x( P.eq_class_representative( otherpiece.smallest() ) );
		assert( x != Invalid );
		// It could be that p is not in part.
		auto State y( P.eq_class_representative( part.smallest() ) );
		assert( y != Invalid );
		assert( x != y );

		if( p == x ) {
			// If p is the representative of `otherpiece', then
			// return the representative of `part'.
			assert( otherpiece.contains( p ) );
			return( y );
		} else {
			// If p is the representative of `part', then return the
			// representative of `otherpiece'.
			assert( part.contains( p ) );
			return( x );
		}
	} else {
		assert( (part == P.equiv_class( p )) || (part.empty()) );
		// No splitting to be done.
		return( Invalid );
	}
}

