/* (c) Copyright 1994 by Bruce Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:18 $
#ifndef RBFA_H
#define RBFA_H

#include "faabs.h"
#include "st-pool.h"
#include "stateset.h"
#include "staterel.h"
#include "trans.h"
#include "rfa.h"
#include <assert.h>
#include <iostream.h>

// Implement right-biased FA's (Construction 4.43 in the Taxonomy).

class RBFA : virtual public FAabs {
public:
	// Constructors, destructors, operator=:

	// Default constructor builds the empty language acceptor.
	RBFA();

	// Default constr, copy constr, destr, operator= are okay.

	// Special constructor (from RFA) implementing R; decode; R (see
        // Construction 4.45 of the Taxonomy):
        RBFA( const RFA& r );


	// Standard FAabs member functions. Don't override acceptable():

        virtual int num_states() const;
	virtual void restart();
	virtual void advance( char a );
	virtual int in_final() const;
	virtual int stuck();
	virtual DFA determinism() const;


	// Some Sigma-algebra conversions:
	//	as the duals of those given in Definitions 4.28 and 4.35
	virtual RBFA& decode( const RFA& r );
	virtual RBFA& convert( const RFA& r );


	// Special member functions:

        friend ostream& operator<<( ostream& os, const RBFA& r );

        inline int class_invariant() const;

protected:
	// Implementation stuff, protected for Reg<RBFA>:

	StatePool Q;
	StateSet S;
        // Single final state as required (Construction 4.43):
	State f;
	// See rfa.h and lbfa.h for explanation of Qmap_inverse:
	Trans Qmap_inverse;
	StateRel follow;

	// Simulation stuff:
        StateSet current;
};

inline int RBFA::class_invariant() const {
	return( Q.contains( f )
		&& Q.size() == S.domain()
                && Q.size() == Qmap_inverse.range()
		&& Q.size() == follow.domain()
		&& Q.size() == current.domain()
		&& S.class_invariant()
		&& Qmap_inverse.class_invariant()
		&& follow.class_invariant()
		&& current.class_invariant() );
}

#endif
