/* (c) Copyright 1994 by Bruce Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:28 $
#ifndef RFA_H
#define RFA_H

#include "faabs.h"
#include "st-pool.h"
#include "stateset.h"
#include "staterel.h"
#include "trans.h"
#include "re.h"
#include "dfa.h"
#include <assert.h>
#include <iostream.h>

// The existence of LBFA and RBFA are needed, for friendship.
// Can't include lbfa.h or rbfa.h due to circularity.
class LBFA;
class RBFA;

// Implement RFA's (reduced finite automata -- Definition 4.24 of the Taxonomy)
// as a type of FAabs.

class RFA : virtual public FAabs {
public:
	// Some constructors:

	RFA();

	// Need a copy constructor:
	RFA( const RFA& r );

	// The Sigma-homomorphism constructor (see Definition 4.30):
	RFA( const RE& e );

	// Default destr, operator= are okay.

	// Normal FAabs member functions (don't override acceptable()):

        virtual int num_states() const;
	virtual void restart();
	virtual void advance( char a );
	virtual int in_final() const;
	virtual int stuck();
	virtual DFA determinism() const;


	// An alternate implementation of determinism() using the LBFA interpretation.
	virtual DFA determinism2() const;


	// Special members:

	friend ostream& operator<<( ostream& os, const RFA& r );

	// These two must be friends so that they can construct themselves properly.
	friend class LBFA;
	friend class RBFA;

	inline int class_invariant() const;

protected:
	// A helper: compute all of the components into place.
	//	for use in the constructor.
	void rfa_into( const RE& e,
		StatePool& Qp,
		StateSet& f,
		StateSet& l,
		Trans& Qm,
		StateRel& foll,
		int& N );

	// Recycle *this:
	void reincarnate();

	// Implementation details, protected for Reg<RFA> use.
	StatePool Q;
	StateSet first, last;
	Trans Qmap_inverse;
	StateRel follow;
	int Nullable;

	// Some simulation stuff.
	int final;
        StateSet current;
};

inline int RFA::class_invariant() const {
	return( Q.size() == first.domain()
		&& Q.size() == last.domain()
                && Q.size() == Qmap_inverse.range()
		&& Q.size() == follow.domain()
		&& Q.size() == current.domain()
		&& first.class_invariant()
		&& last.class_invariant()
		&& Qmap_inverse.class_invariant()
		&& follow.class_invariant()
		&& current.class_invariant() );
}

#endif
