/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:31 $
#include "charrang.h"
#include "reops.h"
#include "re.h"
#include "sigma.h"

// Implementations of the Sigma-algebra operators, with RE as the carrier.
// This is the Sigma-term algebra (see the Taxonomy, Section 3).

Reg<RE>& Reg<RE>::epsilon() {
	// This may have been something in a previous life.
	reincarnate();

	set_root_operator( EPSILON );
        assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::empty() {
	// See epsilon() case.
	reincarnate();

	set_root_operator( EMPTY );
        assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::symbol( const CharRange r ) {
	// See epsilon case.
	reincarnate();

	set_root_operator( SYMBOL );
	set_symbol( r );
        assert( class_invariant() );
        return( *this );
}

Reg<RE>& Reg<RE>::or( const Reg<RE>& r ) {
	assert( class_invariant() );
        assert( r.class_invariant() );

	auto RE *const lft( new RE );

        // Make a copy of *this.
        shallow_copy( lft );
	auto RE *const rt( new RE( r ) );
	op = OR;
	left = lft;
	right = rt;

	assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::concat( const Reg<RE>& r ) {
	assert( class_invariant() );
	assert( r.class_invariant() );

	auto RE *const lft( new RE );
        shallow_copy( lft );
	auto RE *const rt( new RE( r ) );
	op = CONCAT;
	left = lft;
	right = rt;

	assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::star() {
	assert( class_invariant() );

	auto RE *const d( new RE );
        shallow_copy( d );
	op = STAR;
	left = d;
	right = 0;

	assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::plus() {
	assert( class_invariant() );

	auto RE *const d( new RE );
        shallow_copy( d );
	op = PLUS;
	left = d;
	right = 0;

	assert( class_invariant() );
	return( *this );
}

Reg<RE>& Reg<RE>::question() {
	assert( class_invariant() );

	auto RE *const d( new RE );
	shallow_copy( d );
	op = QUESTION;
	left = d;
	right = 0;

	assert( class_invariant() );
	return( *this );
}
