/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 15:59:46 $
#include "staterel.h"

StateSet StateRel::image( const StateSet& r ) const {
	assert( class_invariant() );
        assert( r.class_invariant() );
        assert( r.domain() == domain() );
	// result is, by default, the emptyset.
	auto StateSet result;
        result.set_domain( domain() );
	auto State st;
	for( r.iter_start( st ); !r.iter_end( st ); r.iter_next( st ) ) {
		result.set_union( lookup( st ) );
	}
        return( result );
}

StateSet StateRel::closure( const StateSet& r ) const {
	assert( class_invariant() );
        assert( r.class_invariant() );
	assert( r.domain() == domain() );

	// Use an iterative method to compute the Kleene closure.
	auto StateSet result( r );

	auto StateSet intermediate( image( r ) );
	assert( intermediate.domain() == result.domain() );
	while( !result.contains( intermediate ) ) {
		result.set_union( intermediate );
		intermediate = image( result );
	}
	return( result );
}

// Map all members of S to st as well.
StateRel& StateRel::union_cross( const StateSet& S, State st ) {
	assert( class_invariant() );
        assert( S.class_invariant() );
        assert( (S.domain() == domain()) && (0 <= st) && (st < domain()) );
	auto State i;
	for( S.iter_start( i ); !S.iter_end( i ); S.iter_next( i ) ) {
        	map( i ).add( st );
	}
	assert( class_invariant() ); 
	return( *this );
}

// This could probably have made use of union_cross(State,StateSet&).
StateRel& StateRel::union_cross( const StateSet& A, const StateSet& B ) {
	assert( class_invariant() );
        assert( A.class_invariant() && B.class_invariant() );
	assert( (A.domain() == domain()) && (B.domain() == domain()) );
	auto State i;
	for( A.iter_start( i ); !A.iter_end( i ); A.iter_next( i ) ) {
		map( i ).set_union( B );
	}
        assert( class_invariant() );
	return( *this );
}

StateRel& StateRel::remove_pairs( const StateSet& P, const StateSet& Q ) {
	assert( class_invariant() );
	assert( P.domain() == domain() );
	assert( Q.domain() == domain() );

	auto State i;
	for( P.iter_start( i ); !P.iter_end( i ); P.iter_next( i ) ) {
		map( i ).remove( Q );
	}
	assert( class_invariant() );
	return( *this );
}

StateRel& StateRel::set_union( const StateRel& r ) {
	assert( class_invariant() && r.class_invariant() );
	assert( domain() == r.domain() );
	auto State i;
	for( i = 0; i < r.domain(); i++ ) {
        	map( i ).set_union( r.lookup( i ) );
		// Could also have been done with
		//	union_cross( i, r.lookup( i ) );
        }
	assert( class_invariant() );
        return( *this );
}

void StateRel::set_domain( const int r ) {
	assert( class_invariant() );
	assert( r >= domain() );
	StateTo<StateSet>::set_domain( r );
	auto int i;
	for( i = 0; i < domain(); i++ ) {
        	map( i ).set_domain( r );
	}
        assert( class_invariant() );
}

void StateRel::clear() {
	assert( class_invariant() );
	auto int i;
	for( i = 0; i < domain(); i++ ) {
        	map( i ).clear();
	}
        assert( class_invariant() );
}

void StateRel::reincarnate() {
	assert( class_invariant() );
	auto int i;
	for( i = 0; i < domain(); i++ ) {
		map( i ).reincarnate();
        }
	StateTo<StateSet>::reincarnate();
        assert( class_invariant() );
}

StateRel& StateRel::disjointing_union( const StateRel& r ) {
	assert( class_invariant() && r.class_invariant() );
	auto int olddom( domain() );
	StateTo<StateSet>::disjointing_union( r );
	auto int i;
	// domain() is the new one, olddom is the old one:
	assert( domain() == olddom + r.domain() );

	// Just adjust the domains of those StateSet's that were already in *this.
	for( i = 0; i < olddom; i++ ) {
		map( i ).set_domain( domain() );
	}
        // Got to rename the incoming ones.
	for( ; i < domain(); i++ ) {
		map( i ).st_rename( olddom );
	}
	assert( class_invariant() );
	return( *this );
}

ostream& operator<<( ostream& os, const StateRel& r ) {
	assert( r.class_invariant() );
	return( os << (const StateTo<StateSet>&)r );
}

