/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 16:00:35 $
#include "tr-impl.h"

// Copy constructor allocates more memory. Should use copy-on-write (use-
// counting) for efficiency.
TransImpl::TransImpl( TransImpl& const r ) :
		howmany( r.in_use + expansion_extra ),
		in_use( r.in_use ),
		data( new TransPair [r.in_use + expansion_extra] ) {
	assert( r.class_invariant() );
	auto int i;
	for( i = 0; i < in_use; i++ ) {
		data[i] = r.data[i];
	}
        assert( class_invariant() );
}
                                  
// Destructor is virtual, simple.
virtual TransImpl::~TransImpl() {
	assert( class_invariant() );
	delete [] data;
}

// Standard dynamic memory copy:
const TransImpl& TransImpl::operator=( const TransImpl& r ) {
	assert( class_invariant() && r.class_invariant() );
	if( this != &r ) {
        	// Don't use ensure_min_size() here due to excessive copying.
		if( howmany < r.in_use ) {
			auto TransPair *d( new TransPair [howmany = r.in_use
					+ expansion_extra] );
			delete data;
                        data = d;
		}
                assert( howmany >= r.in_use );
		auto int i;
		for( i = 0; i < r.in_use; i++ ) {
			data[i] = r.data[i];
		}
		in_use = r.in_use;
	}
        assert( class_invariant() );
	return( *this );
}

CRSet TransImpl::out_labels() const {
	auto int i;
        auto CRSet a;
	for( i = 0; i < in_use; i++ ) {
        	a.add( data[i].transition_label );
	}
        return( a );
}

CRSet TransImpl::labels_into( const StateSet& r ) const {
	assert( class_invariant() );
        assert( r.class_invariant() );
	auto int i;
	auto CRSet a;
	for( i = 0; i < in_use; i++ ) {
		assert( data[i].transition_destination <= r.domain() );
		if( r.contains( data[i].transition_destination ) ) {
                	a.add( data[i].transition_label );
                }
	}
        assert( class_invariant() );
        return( a );
}

TransImpl& TransImpl::add_transition( const CharRange a, const State q ) {
	assert( 0 <= q );
	assert( a.class_invariant() );

	// Can the transition be merged with an already existing one.
	auto int i;
	for( i = 0;  i < in_use; i++ ) {
		// They're mergeable if they go to the same place
		// and the labels are adjacent.
		if( (data[i].transition_destination == q)
				&& (data[i].transition_label.overlap_or_adjacent( a )) ) {
			data[i].transition_label.merge( a );
			return( *this );
		}
	}
	// The transition really must be added, since it wasn't merged.

	// Is there a need to grow the data[].
	ensure_min_size( in_use + 1 );

	assert( in_use < howmany );
	data[in_use].transition_label = a;
	data[in_use++].transition_destination = q;

	return( *this );
}

void TransImpl::ensure_min_size( int w ) {
	assert( class_invariant() );
	assert( 0 <= w );
	if( howmany < w ) {
		auto TransPair *d( new TransPair [howmany = w + expansion_extra] );
		auto int i;
		for( i = 0; i < in_use; i++ ) {
			d[i] = data[i];
		}
		// Assume that delete [] 0 is okay.
		delete [] data;
		data = d;
	}
	assert( howmany >= w );
	assert( class_invariant() );
}

ostream& operator<<( ostream& os, const TransImpl& r ) {
	assert( r.class_invariant() );
	os << '{';
	auto int i;
	for( i = 0; i < r.in_use; i++ ) {
		os << ' ' << r.data[i].transition_label << "->"
			<< r.data[i].transition_destination << ' ';
	}
        return( os << '}' );
}
