/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 16:00:38 $
#ifndef TRANSIMPL_H
#define TRANSIMPL_H

#include "charrang.h"
#include "crset.h"
#include "state.h"
#include "stateset.h"
#include "tr-pair.h"
#include <assert.h>
#include <iostreams.h>

// Implement the basics of a char to StateSet map (a set of TransPair's).
// It is used by DTrans and Trans.

class TransImpl {
protected:
	// Constructors, destructors, operator=:

	// Assume that delete [] 0 is okay.
	inline TransImpl();

	// Copy constructor allocates more memory. Should use (use-counting)
	// for efficiency.
	TransImpl( TransImpl& const r );

        // Destructor is virtual, simple.
	virtual ~TransImpl();

        // operator=() must copy the memory.
	const TransImpl& operator=( const TransImpl& r );


	// Some member functions for making transitions:


	// What are all of the labels on transitions in *this?
	CRSet out_labels() const;

	// What are all transition labels with destination in r?
        CRSet labels_into( const StateSet& r ) const;


	// Some special member functions:

	// Clear out all previous transitions, and zero the domain.
	inline void reincarnate();


	// Add a transition to the set.
	TransImpl& add_transition( const CharRange a, const State q );

	// Allow classes that inherit from TransImpl to have access to the real data:
	inline TransPair& transitions( const int i ) const;

	// Output the transitions.
	friend ostream& operator<<( ostream& os, const TransImpl& r );

	// Maintain the class invariant.
	inline int class_invariant() const;


	// Helpers:
	void ensure_min_size( int w );


	// Implementation details:

	// How many transitions are there.
	int howmany;
        int in_use;

private:
	// A dynamically allocated array of (CharRange,State) pairs (transitions).
	TransPair *data;

	// For efficiency of the expansion helper function.
        enum { expansion_extra = 5 };
};


// Some inline members:

// Assume that delete [] 0 is okay.
inline TransImpl::TransImpl() :
		howmany( 0 ),
		in_use( 0 ),
		data( 0 ) {
	assert( class_invariant() );
}

inline void TransImpl::reincarnate() {
	assert( class_invariant() );
	in_use = 0;
	assert( class_invariant() );
}

inline TransPair& TransImpl::transitions( const int i ) const {
	assert( class_invariant() );
	// i can be greater than in_use in some weird cases where the
	// transitions are still being set up.
	assert( 0 <= i );
	return( data[i] );
}

// Maintain the class invariant.
inline int TransImpl::class_invariant() const {
	auto int ret( 1 );
	auto int i;
	for( i = 0; (i < in_use) && ret; i++ ) {
		ret = (data[i].transition_destination >= 0);
	}
	ret = ret && (in_use <= howmany);
       	return( ret );
}

#endif

