/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 16:00:42 $
#include "trans.h"


StateSet Trans::operator[]( const char a ) const {
	assert( class_invariant() );
	auto int i;
	auto StateSet result;
        result.set_domain( range() );
	for( i = 0; i < in_use; i++ ) {
		if( transitions( i ).transition_label.contains( a ) ) {
			result.add( transitions( i ).transition_destination );
		}
	}
	return( result );
}

StateSet Trans::range_transition( const CharRange a ) const {
	assert( class_invariant() );
	auto int i;
	auto StateSet result;
	result.set_domain( range() );
	for( i = 0; i < in_use; i++ ) {
		if( a <= transitions( i ).transition_label ) {
			result.add( transitions( i ).transition_destination );
                }
	}
        return( result );
}

// The following member does not change the range.
Trans& Trans::set_union( const Trans& r ) {
	assert( class_invariant() && r.class_invariant() );
	assert( range() == r.range() );
	if( r.in_use != 0 ) {
        	// May need to expand here:
		ensure_min_size( in_use + r.in_use );

		assert( howmany >= in_use + r.in_use );
		auto int i;
		for( i = 0; i < r.in_use; i++ ) {
                	transitions(i + in_use) = r.transitions( i );
		}
		in_use += r.in_use;
        }
	assert( class_invariant() );
        return( *this );
}

Trans& Trans::disjointing_union( const Trans& r ) {
	assert( class_invariant() );
	// Don't do any useless work.
	if( r.in_use != 0 ) {
		ensure_min_size( in_use + r.in_use );

		assert( howmany >= in_use + r.in_use );
		auto int i;
		for( i = 0; i < r.in_use; i++ ) {
			transitions( i + in_use ).transition_label =
				r.transitions( i ).transition_label;
			transitions( i + in_use ).transition_destination =
                        	r.transitions( i ).transition_destination + destination_range;
		}
		in_use += r.in_use;
                destination_range += r.destination_range;
	}
        assert( class_invariant() );
	return( *this );
}

// Rename all states in this Trans, so that they don't clash with
// those in the range [0,r).
Trans& Trans::st_rename( const int r ) {
	assert( class_invariant() );
	// Don't do anything useless:
	if( r != 0 ) {
		auto int i;
		for( i = 0; i < in_use; i++ ) {
			transitions( i ).transition_destination += r;
		}
		destination_range += r;
	}
	assert( class_invariant() );
	return( *this );
}

ostream& operator<<( ostream& os, const Trans& r ) {
	assert( r.class_invariant() );
	return( os << (TransImpl&)r );
}
