/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 16:00:46 $
#include "transrel.h"

StateSet TransRel::image( const StateSet& r, const char a ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
        assert( r.domain() == domain() );
	// result is, by default, the emptyset.
	auto StateSet result;
        result.set_domain( domain() );
	auto State st;
	for( r.iter_start( st ); !r.iter_end( st ); r.iter_next( st ) ) {
        	// Here we assume that the Trans in the TransRel have the same domain too.
		result.set_union( lookup( st )[a] );
	}
        return( result );
}

StateSet TransRel::transition_on_range( const StateSet& r, const CharRange a ) const {
	assert( class_invariant() );
	assert( r.class_invariant() );
	assert( r.domain() == domain() );

	auto StateSet result;
        result.set_domain( domain() );
        // Go through all of the States to see which transitions happen.
	auto State st;
	for( r.iter_start( st ); !r.iter_end( st ); r.iter_next( st ) ) {
        	result.set_union( lookup( st ).range_transition( a ) );
	}
        return( result );
}

CRSet TransRel::out_labels( const StateSet& r ) const {
	assert( class_invariant() );

	auto CRSet result;
	auto State st;
        // Go through all of r's States.
	for( r.iter_start( st ); !r.iter_end( st ); r.iter_next( st ) ) {
        	result.combine( lookup( st ).out_labels() );
	}
	return( result ); 
}

void TransRel::set_domain( const int r ) {
	assert( class_invariant() );
	StateTo<Trans>::set_domain( r );
	auto int i;
	for( i = 0; i < domain(); i++ ) {
        	map( i ).set_range( r );
	}
        assert( class_invariant() );
}

void TransRel::reincarnate() {
	assert( class_invariant() );
	auto int i;
	// First, reincarnate() all of the components before *this.
	for( i = 0; i < domain(); i++ ) {
		map( i ).reincarnate();
        }
	StateTo<Trans>::reincarnate();
        assert( class_invariant() );
}

TransRel& TransRel::disjointing_union( const TransRel& r ) {
	assert( class_invariant() && r.class_invariant() );
	auto int olddom( domain() );
	StateTo<Trans>::disjointing_union( r );
	assert( StateTo<Trans>::domain() == r.domain() + olddom );
	auto int i;
	for( i = 0; i < olddom; i++ ) {
		map( i ).set_range( domain() );
        }
	for( ; i < domain(); i++ ) {
		map( i ).st_rename( olddom );
	}
        assert( class_invariant() );
	return( *this );
}

ostream& operator<<( ostream& os, const TransRel& r ) {
	assert( r.class_invariant() );
	return( os << (const StateTo<Trans>&)r );
}
