/* (c) Copyright 1994 by Bruce W. Watson */
// $Revision: 1.1 $
// $Date: 1994/05/02 16:00:48 $
#ifndef TRANSREL_H
#define TRANSREL_H

#include "state.h"
#include "stateset.h"
#include "stateto.h"
#include "charrang.h"
#include "crset.h"
#include "trans.h"
#include <iostream.h>

// Implement a transition relation, using a function from States to Trans (which, in turn
// are char -> State).
// This is used for transition relations in normal FA's.

class TransRel : protected StateTo<Trans> {
public:
	// Constructors, destructors, operator=:

	// Default argument-less constructor is okay.
	inline TransRel();
	
	inline TransRel( const TransRel& r );

        // Default destructor is okay

	inline const TransRel& operator=( const TransRel& r );


	// Some relational operators:

        // Compute the image of r, and a under *this.
	StateSet image( const StateSet& r, const char a ) const;

	// Transitioning on a CharRange? (Same idea as above.)
	StateSet transition_on_range( const StateSet& r, const CharRange a ) const;

	// On which labels can we transition?
        CRSet out_labels( const StateSet& r ) const;


	// Some functions updating *this:

	inline TransRel& add_transition( const State p, const CharRange r, const State q );


	// Some domain related members:

        // What is the domain of this relation.
	inline int domain() const;

        // Change the domain of this relation.
	void set_domain( const int r );

	// Recycle this entire structure.
	void reincarnate();

        // Union relation r into *this, while adjusting r.
	TransRel& disjointing_union( const TransRel& r );


	// Some special members:

	friend ostream& operator<<( ostream& os, const TransRel& r );

	inline int class_invariant() const;
};

inline TransRel::TransRel() :
		StateTo<Trans>() {
	assert( class_invariant() );
}

inline TransRel::TransRel( const TransRel& r ) :
		StateTo<Trans>( (StateTo<Trans> &)r ) {
	assert( class_invariant() );
}

inline const TransRel& TransRel::operator=( const TransRel& r ) {
	assert( r.class_invariant() );
	// Call back to the base class.
        StateTo<Trans>::operator=( (const StateTo<Trans>&) r );
	assert( class_invariant() );
	return( *this );
}

inline int TransRel::domain() const {
	return( StateTo<Trans>::domain() );
}

inline TransRel& TransRel::add_transition( const State p, const CharRange r, const State q ) {
	assert( class_invariant() );
	assert( (0 <= p) && (p < domain()) );
	assert( (0 <= q) && (q < domain()) );
        map( p ).add_transition( r, q );
	assert( class_invariant() );
        return( *this );
}

inline int TransRel::class_invariant() const {
	// First, we must satisfy the base class_invariant().
	auto int res( StateTo<Trans>::class_invariant() );
	auto int i;
	for( i = 0; (i < domain()) && res; i++ ) {
		res = lookup( i ).class_invariant() && (lookup( i ).range() == domain());
        }
	return( res );
}

#endif

